


/*
-----------------------------------------------------------------------------
This source file is part of OGRE
    (Object-oriented Graphics Rendering Engine)
For the latest info, see http://www.ogre3d.org/

Copyright (c) 2000-2005 The OGRE Team
Also see acknowledgements in Readme.html

You may use this sample code for anything you like, it is not covered by the
LGPL like the rest of the engine.
-----------------------------------------------------------------------------
*/

/**
    \file 
        Lighting.h
    \brief
        Shows lighting support in OGRE. Also demonstrates the use of billboards
        and controllers for automatic time-relative behaviour.
*/


#include "ExampleApplication.h"
#include "OgreIlluminationManager.h"

float causize = 10;
float cauintens = 40;

// Listener class for frame updates
class RaytraceDemoListener : public ExampleFrameListener
{
protected:
	
	Light* light;
	SceneNode* lightNode;
	SceneNode* sphereNode;
	unsigned long framecount;
	
public:
    RaytraceDemoListener(RenderWindow* window, Camera* maincam,SceneNode* lightNode, SceneNode* sphereNode, Light* l)
        :ExampleFrameListener(window, maincam) 
				 
    {
		framecount = 0;
		this->sphereNode = sphereNode;
		this->lightNode = lightNode;
		light = l;
    }
	bool processUnbufferedKeyInput(const FrameEvent& evt)
    {
		if (mInputDevice->isKeyDown(KC_SUBTRACT))
        {
			Vector3 lPos = lightNode->getPosition();
			Vector3 lDir = sphereNode->getPosition() - lPos;
			lDir.normalise();
			lPos -= lDir;
			lightNode->setPosition(lPos);
			light->setDirection(lDir);
		}
		if (mInputDevice->isKeyDown(KC_ADD))
        {
			
			Vector3 lPos = lightNode->getPosition();
			Vector3 lDir = sphereNode->getPosition() - lPos;
			lDir.normalise();
			lPos += lDir;
			lightNode->setPosition(lPos);
			light->setDirection(lDir);
		}
		if (mInputDevice->isKeyDown(KC_MULTIPLY))
        {
			Vector3 lPos = lightNode->getPosition();
			Vector3 lDir = sphereNode->getPosition() - lPos;
			lDir.normalise();
			Vector3 right = lDir.crossProduct(Vector3(0,1,0));
			Vector3 up = lDir.crossProduct(right);
			lPos += up,
			lightNode->setPosition(lPos);
			light->setDirection(lDir);
		}
		if (mInputDevice->isKeyDown(KC_DIVIDE))
        {
			Vector3 lPos = lightNode->getPosition();
			Vector3 lDir = sphereNode->getPosition() - lPos;
			lDir.normalise();
			Vector3 right = lDir.crossProduct(Vector3(0,1,0));
			Vector3 up = lDir.crossProduct(right);
			lPos -= up,			
			lightNode->setPosition(lPos);
			light->setDirection(lDir);
				
		}
		if (mInputDevice->isKeyDown(KC_HOME))
        {
			Vector3 lPos = lightNode->getPosition();
			Vector3 lDir = sphereNode->getPosition() - lPos;
			lDir.normalise();
			Vector3 right = lDir.crossProduct(Vector3(0,1,0));
			Vector3 up = lDir.crossProduct(right);
			lPos += right,
			lightNode->setPosition(lPos);
			light->setDirection(lDir);
		}
		if (mInputDevice->isKeyDown(KC_END))
        {
			Vector3 lPos = lightNode->getPosition();
			Vector3 lDir = sphereNode->getPosition() - lPos;
			lDir.normalise();
			Vector3 right = lDir.crossProduct(Vector3(0,1,0));
			Vector3 up = lDir.crossProduct(right);
			lPos -= right,			
			lightNode->setPosition(lPos);
			light->setDirection(lDir);
				
		}
		return ExampleFrameListener::processUnbufferedKeyInput(evt);
	}
    bool frameStarted(const FrameEvent& evt)
    {
		static bool first = true;
	

	//	lightNode->setPosition( 100.0 * Vector3(cos(-(float)framecount/500.0),2.0,sin(-(float)framecount/500.0)));
		
		
		bool result = ExampleFrameListener::frameStarted(evt);
       framecount++;

	   if (framecount == 60)
		   OgreIlluminationManager::getSingleton().freeAllResources();
      return result;     
    }

	

};

/** Application class */
class RaytraceDemoApplication : public ExampleApplication
{

public:

protected:
	SceneNode* mainLightNode;
	SceneNode* sphereNode;
	
	void chooseSceneManager()
	{
		//mSceneMgr = mRoot->createSceneManager("OctreeSceneManager");
		mSceneMgr = mRoot->createSceneManager("OcclusionCullingSceneManager");
	}

	void loadResources(void)
	{
		
		// Initialise, parse scripts etc
		ResourceGroupManager::getSingleton().initialiseResourceGroup("Bootstrap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("General");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Basic");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("PostProc");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_EnvMap");
		ResourceGroupManager::getSingleton().initialiseResourceGroup("GTP_Caustic");
	}

	void createPostproc()
	{		
		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/Glow");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/Glow", true);

		CompositorManager::getSingleton().addCompositor(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap");
		CompositorManager::getSingleton().setCompositorEnabled(mWindow->getViewport(0),
														"GTP/PostProc/ToneMap", true);		
	}
    
	Light* light;
	void createScene(void)
    {
		std::string occlusion_cfg("OcclusionCulling.cfg");
		mSceneMgr->setWorldGeometry(occlusion_cfg);

		//createPostproc();
		OgreIlluminationManager::getSingleton().setBlurShadowMap(true);
		OgreIlluminationManager::getSingleton().setUseLISPSM(false);

		OgreIlluminationManager::getSingleton().setMainCamera(mCamera);
		OgreIlluminationManager::getSingleton().setMainViewport(mWindow->getViewport(0));
		
		OgreIlluminationManager::getSingleton().setMaxJoinRadius(420);		

		Root::getSingleton()._setCurrentSceneManager(mSceneMgr);

		mCamera->setPosition(0,100,240);
		mCamera->setFOVy(Radian(Degree(80)));
		//mCamera->setFarClipDistance(1000);
		// Set ambient light
        mSceneMgr->setAmbientLight(ColourValue(1, 1, 1));

		// Skybox
        mSceneMgr->setSkyBox(true, "Examples/EveningSkyBox");

		SceneNode* rootNode = mSceneMgr->getRootSceneNode();

      	Entity* sphere = mSceneMgr->createEntity("sphere", "head.mesh");
		
	//	Material* mat = (Material*) MaterialManager::getSingleton().getByName("GTP/Caustic/Glass").getPointer();
	//	Material* newMat = mat->clone(mat->getName() + "_new").getPointer();

		sphere->setMaterialName("GTP/Caustic/Glass");
		sphereNode = rootNode->createChildSceneNode();
		//sphereNode->setScale(0.1, 0.1, 0.1);//sphere
		sphereNode->setPosition(0, 50, 0);
		sphereNode->attachObject(sphere);
		sphereNode->_updateBounds();

		Entity* room = mSceneMgr->createEntity("room", "atlascube.mesh");
	//	mat = (Material*) MaterialManager::getSingleton().getByName("TestPlane").getPointer();
	//	newMat = mat->clone(mat->getName() + "_new").getPointer();

		room->setMaterialName("TestPlane");
		SceneNode *roomNode = rootNode->createChildSceneNode();
		//roomNode->setScale(0.1, 0.1, 0.1);
		roomNode->setPosition(0, 50, 0);
		roomNode->attachObject(room);
		

		Light* mainLight = mSceneMgr->createLight("MainLight");
		light = mainLight;
		mainLight->setType(Light::LT_POINT);
        mainLight->setDiffuseColour(ColourValue::White);
		mainLight->setSpecularColour(ColourValue::White);
		mainLightNode = rootNode->createChildSceneNode();
		mainLightNode->setPosition(100,100,0);
		//mainLight->setPosition(mainLightNode->getWorldPosition());
		mainLightNode->attachObject(mainLight); 
		BillboardSet* bbs = mSceneMgr->createBillboardSet("bb", 1);
		bbs->setDefaultDimensions(10,10);
		bbs->createBillboard(Vector3::ZERO, ColourValue::White);
		bbs->setMaterialName("Flare");
		mainLightNode->attachObject(bbs);

		//createPlane("ground", "TestPlane", Vector3(0,0,0), Vector2(500,500));
		OgreIlluminationManager::getSingleton().initTechniques();
	 }
   void createPlane(Ogre::String _name, Ogre::String _material, Ogre::Vector3 _pos, Ogre::Vector2 _size, Ogre::Vector2 _subdivisions = Ogre::Vector2(1,1)) {
		
	   Plane _plane;
        _plane.normal = Vector3::UNIT_Y;
        _plane.d = 0;


        MeshManager::getSingleton().createPlane(_name + ".plane",
            ResourceGroupManager::DEFAULT_RESOURCE_GROUP_NAME, 
            _plane,_size.x,_size.y,1,1,true,1,_subdivisions.x,_subdivisions.y,Vector3::UNIT_Z);
        
		Entity *_entity = mSceneMgr->createEntity(_name + ".entity", _name + ".plane");
        _entity->setMaterialName(_material);

		SceneNode* _node = mSceneMgr->getRootSceneNode()->createChildSceneNode(_name + ".node");
		_node->attachObject(_entity);
		_node->setPosition(_pos);
	}



	void createFrameListener(void)
    {
		// This is where we instantiate our own frame listener
		mFrameListener= new RaytraceDemoListener(mWindow,
											mCamera,
											mainLightNode,
											sphereNode,
											light
											);
		mFrameListener->setPriority(1);
		OgreIlluminationManager::getSingleton().setPriority(2);
        mRoot->addFrameListener(&OgreIlluminationManager::getSingleton());
		mRoot->addFrameListener(mFrameListener);
	
    }

};

